"""Agent framework integrations.

Available integrations:
- kameleondb.integrations.mcp - MCP (Model Context Protocol) server

Planned integrations:
- kameleondb.integrations.langchain - LangChain tools
- kameleondb.integrations.openai - OpenAI function calling
- kameleondb.integrations.crewai - CrewAI tools
"""
